import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/material.dart';
import 'package:prod_scheduler_acc/constants.dart';
import 'package:prod_scheduler_acc/models/pending_events_datasource.dart';
import 'package:prod_scheduler_acc/models/scheduler_event.dart';
import 'package:prod_scheduler_acc/models/scheduler_user.dart';
import 'package:prod_scheduler_acc/scheduler_utils.dart';
import 'package:syncfusion_flutter_datagrid/datagrid.dart';
import 'package:syncfusion_flutter_core/theme.dart';

class PendingGrid extends StatefulWidget {
  final SchedulerUser _inputCurrentUser;
  const PendingGrid({
    Key? key,
    required SchedulerUser inputCurrentUser,
  })  : _inputCurrentUser = inputCurrentUser,
        super(key: key);

  @override
  State<PendingGrid> createState() => _PendingGridState();
}

class _PendingGridState extends State<PendingGrid> {
  /// to set the data to the data grid
  /// initialize it within initState
  late Stream<QuerySnapshot> _pendingEventsStream;
  List<SchedulerEvent> pendingEvents = [];
  late PendingEventDataSource _pendingEventDataSource;
  late SchedulerUser _inputCurrentUser;
  final DataGridController _dataGridController = DataGridController();
  bool _eventsListIsEmpty = false;

  /// key used to try a force refresh
  final GlobalKey<_PendingGridState> key = GlobalKey<_PendingGridState>();

  @override
  void initState() {
    /// Get the data with a reference to the Firestore
    _pendingEventsStream = FirebaseFirestore.instance.collection(kQptFireStoreEventCollection).where(kQptIntFldIsPending, isEqualTo: true).snapshots();

    /// grab the user
    _inputCurrentUser = widget._inputCurrentUser;
    super.initState();
  }

  @override
  void dispose() {
    _dataGridController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return StreamBuilder<QuerySnapshot>(
        stream: _pendingEventsStream,
        builder: (context, fireStoreSnapshot) {
          if (fireStoreSnapshot.hasError) {
            /// TODO: have this placed in the proper location rather than this implementation
            return Text('Something went wrong: ${fireStoreSnapshot.error}');
          }
          if (fireStoreSnapshot.connectionState == ConnectionState.waiting) {
            return const Center(child: CircularProgressIndicator());
          }
          if (fireStoreSnapshot.hasData) {
            /// the clear call is used to refresh the data after an update
            pendingEvents.clear();
            final pendingDocsFromFireStore = fireStoreSnapshot.data!.docs;
            if (pendingDocsFromFireStore.isNotEmpty) {
              for (var pendingEvent in pendingDocsFromFireStore) {
                pendingEvents.add(SchedulerEvent.fromFireStore(pendingEvent as DocumentSnapshot<Map<String, dynamic>>));
              }
            } else {
              /// set a flag that will show no data
              _eventsListIsEmpty = true;
            }

            /// now that the collection of pending events has been built,
            /// add the datagrid and set the list of Pending Events into the PendingEventDataSource(pendingEvents) as source
            return SfDataGridTheme(
              data: SfDataGridThemeData(headerColor: kQptColorPrimaryDarkTeal, gridLineColor: kQptColorTeal),
              child: Row(
                children: [
                  Visibility(
                    visible: _eventsListIsEmpty,
                    child: Expanded(
                      child: Card(
                        elevation: 5,
                        margin: const EdgeInsets.all(5.0),
                        child: Row(
                          children: const [
                            Expanded(
                              flex: 1,
                              child: SizedBox(
                                height: 35,
                                width: 35,
                                child: Icon(color: Colors.amber, Icons.warning_amber),
                              ),
                            ),
                            Expanded(
                              flex: 10,
                              child: Text(textAlign: TextAlign.center, 'There are currently no Pending Burial Events.\n To add one, click on the button below'),
                            ),
                            Expanded(
                              flex: 1,
                              child: SizedBox(
                                height: 35,
                                width: 35,
                                child: Icon(color: Colors.amber, Icons.warning_amber),
                              ),
                            ),
                          ],
                        ),
                      ),
                    ),
                  ),
                  Visibility(
                    visible: !_eventsListIsEmpty,
                    child: Expanded(
                      child: SfDataGrid(
                        controller: _dataGridController,
                        selectionMode: SelectionMode.single,
                        source: _pendingEventDataSource = PendingEventDataSource(pendingEvents),
                        columnWidthMode: ColumnWidthMode.fill,
                        columns: [
                          GridColumn(
                            columnName: kQptIntFldCemeteryLocation,
                            label: Container(
                              alignment: Alignment.center,
                              padding: const EdgeInsets.symmetric(horizontal: 16.0),
                              child: const Text(
                                'Requested Location',
                                style: TextStyle(color: Colors.white),
                              ),
                            ),
                          ),
                          GridColumn(
                            columnName: 'fname',
                            label: Container(
                              alignment: Alignment.center,
                              padding: const EdgeInsets.symmetric(horizontal: 16.0),
                              child: const Text(
                                'First Name',
                                style: TextStyle(color: Colors.white),
                              ),
                            ),
                          ),
                          GridColumn(
                            columnName: 'lname',
                            label: Container(
                              alignment: Alignment.center,
                              padding: const EdgeInsets.symmetric(horizontal: 16.0),
                              child: const Text(
                                'Last Name',
                                style: TextStyle(color: Colors.white),
                              ),
                            ),
                          ),
                          GridColumn(
                            columnName: 'created',
                            label: Container(
                              alignment: Alignment.center,
                              padding: const EdgeInsets.symmetric(horizontal: 16.0),
                              child: const Text(
                                'Created',
                                style: TextStyle(color: Colors.white),
                              ),
                            ),
                          ),
                        ],
                        onSelectionChanging: onRowSelecting,
                        onSelectionChanged: onRowSelected,
                      ),
                    ),
                  ),
                ],
              ),
            );
          } // hasData
          else {
            /// TODO: handle no data coming back gracefully...
            return const Text("No Pending Event Data");
          }
        });
  }

  /// This callback is raised while selecting a row at the execution time before the row is selected.
  /// So it allows canceling the selection action by return false.
  bool onRowSelecting(List<DataGridRow> addedRows, List<DataGridRow> removedRows) {
    //final index = _pendingEventDataSource.dataGridRows.indexOf(addedRows.last);
    //SchedulerEvent selectedPendingEvent = pendingEvents[index];
    //print('row selecting: ${selectedPendingEvent.fname}');
    return true;
  }

  /// This callback is raised after the row is selected
  Future<void> onRowSelected(List<DataGridRow> addedRows, List<DataGridRow> removedRows) async {
    //ScaffoldMessengerState scaffoldState = ScaffoldMessenger.of(context);

    final index = _pendingEventDataSource.dataGridRows.indexOf(addedRows.last);
    SchedulerEvent selectedPendingEvent = pendingEvents[index];
    final submitStatus = await SchedulerUtils.openReadDialog(context, selectedPendingEvent, "${_inputCurrentUser.fname} ${_inputCurrentUser.lname}");
    /*ScaffoldFeatureController scaffoldController = scaffoldState.showSnackBar(
      SnackBar(
        duration: const Duration(seconds: 1),
        content: Text(submitStatus ?? "Processing..."),
      ),
    );*/
    //clear the selection so that the user can come in and choose the same row again
    _dataGridController.selectedIndex = -1;
  }
}
